/** \fn get_secure_context(std::string& filename, std::string *keyid, uint16_t *algo)
* @brief This function retrieves the key ID and the algorithm used to encrypt the file specified by filename.
* @param [in] filename Absolute or relative path of the file.
* @param [out] keyid The ID of the key used to encrypt the file.
* @param [out] algo The algorithm used to encrypt the file.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t secure_mkdir(std::string& path, L1 *SEcubeptr, uint32_t key)
* @brief This function creates a directory with an encrypted name.
* @param [in] path Absolute or relative path of the new directory.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @param [in] key ID of the key to be used to encrypt the name of the new directory.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t secure_ls(std::string& path, std::vector<std::pair<std::string, std::string>>& list, L1 *SEcubeptr)
* @brief This function identifies which encrypted files and encrypted directories are present in the directory pointed by path and writes them in list.
* @param [in] path Absolute or relative path to the directory to browse.
* @param [out] list List of pairs containing the encrypted name and the decrypted name.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @return The function returns a 0 in case of success. See \ref errorValues for error list.
* @details Notice that, if the name of a file or of a directory belonging to the path is not associated to SEfile, then it is copied as it is in the list. This function is not recursive. */

/** \fn uint16_t secure_getfilesize(char *path, uint32_t * position, L1 *SEcubeptr)
* @brief This function is used to get the total logic size of an encrypted file pointed by path. Logic size will always be smaller than physical size because it takes into account the overhead introduced by SEfile.
* @param [in] path Absolute or relative path the file.
* @param [out] position Where the size of the file is stored.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn secure_recrypt(std::string path, uint32_t key, L1 *SEcubeptr)
* @brief This function re-encrypts an encrypted file pointed by path with the new key specified as parameter.
* @param [in] path Absolute or relative path of the file.
* @param [in] key The ID of the key used to encrypt the file.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @return The function returns 0 in case of success. See \ref errorValues for error list.
* @details This function should be used to re-encrypt a file that was encrypted with a key that is not trusted anymore (i.e. a compromised key). */

/** \fn uint16_t decrypt_filename(std::string& path, char *filename, L1 *SEcubeptr)
* @brief This function is used to compute the plaintext of an encrypted filename stored in path.
* @param [in] path Where the encrypted file is stored, it can be an absolute or relative path. No encrypted directory names are allowed.
* @param [out] filename A preallocated string where to store the plaintext filename.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t decrypt_dirname(std::string& path, char *decDirname, L1 *SEcubeptr)
* @brief This function is used to compute the plaintext of am encrypted directory name stored in dirname.
* @param [in] dirpath Path to the directory whose name has to be decrypted.
* @param [out] decDirname A preallocated char array where to store the decrypted directory name.
* @param [in] SEcubeptr Pointer to the L1 object used to communicate with the SEcube.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t valid_directory_name(std::string& name)
 * @brief This function checks if the given name can be a valid SEfile directory name.
 * @param [in] name Name of the directory.
 * @return The function returns 0 in case of success. See \ref errorValues for error list.
 * @details This function takes as input the name of a directory and checks if it matches the
 * requirements of an encrypted directory name create with secure_mkdir() (i.e. alphanumeric characters,
 * minimum number of characters, etc.).*/

/** \fn  uint16_t valid_file_name(std::string& name)
 * @brief This function checks if the given name can be a valid SEfile file name.
 * @param [in] name Name of the file.
 * @return The function returns 0 in case of success. See \ref errorValues for error list.
 * @details This function takes as input the name of a file and checks if it matches the
 * requirements of an encrypted file name created with crypto_filename() (i.e. alphanumeric characters,
 * minimum number of characters, etc.).*/

/**  \fn uint16_t SEfile::secure_init(L1 *l1ptr, uint32_t keyID, uint16_t crypto)
* @brief This function is used to initialize the security context of a SEfile object.
* @return The function returns a (uint16_t) '0' in case of success. See \ref errorValues for error list.
* @details Notice that this function works similarly to the SEfile constructor with the same parameters.
* 		   If you specify a key that is not stored in the SEcube or an algorithm that is not supported by
* 		   SEfile, this function will return an error. */

/** \fn void SEfile::secure_finit()
*  @brief This function resets the parameters set by the secure_init() to default values (0s and NULL). */

/** \fn uint16_t SEfile::secure_open(char *path, int32_t mode, int32_t creation)
* @brief This function opens or creates a secure file managed with SEfile.
* @param [in] path The absolute/relative path (plaintext, i.e. myfile.txt) where to open or create the file.
* @param [in] mode The mode in which the file should be opened. See \ref Mode_Defines.
* @param [in] creation Define if the file should be created or it should already exist. See \ref Creation_Defines.
* @return The function returns 0 in case of success. See \ref errorValues for error list.
* @details Notice that you must specify if you want to open the file or if you want to create it. There is not the
* possibility of "create if the file does not exist", therefore if you need such behaviour you must check in advance
* if the file exists or not (i.e. computing the encrypted name of the file with crypto_filename() and then checking
* with OS system calls if the file exists). */

/** \fn uint16_t SEfile::secure_close()
* @brief This function releases the resources related to the underlying SEfile object (i.e. closes the file descriptor).
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_read(uint8_t *dataOut, uint32_t dataOut_len, uint32_t *bytesRead)
* @brief This function reads dataOut_len bytes into dataOut from the file descriptor managed by the underlying SEfile object.
* @param [out] dataOut An already allocated array of characters where to store data read.
* @param [in] dataOut_len Number of characters we want to read.
* @param [out] bytesRead Number of effective characters read, it cannot be NULL.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_write(uint8_t *dataIn, uint32_t dataIn_len)
* @brief This function writes the bytes stored at dataIn to the encrypted file managed by the SEfile object on which this method is called.
* @param [in] dataIn The array of bytes that have to be written.
* @param [in] dataIn_len The length, in bytes, of the data that have to be written.
* @return The function returns a 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_seek(int32_t offset, int32_t *position, uint8_t whence)
* @brief This function is used to move the file pointer of a file managed by a SEfile object.
* @param [in] offset Amount of bytes we want to move.
* @param [out] position Pointer to a int32_t variable where the final position is stored, it cannot be NULL.
* @param [in] whence According to this parameter we can choose if we want to move from the file beginning, file ending or current file pointer position. See \ref Seek_Defines.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_truncate(uint32_t size)
* @brief This function resizes the file managed by the underlying SEfile object to size bytes. If size is bigger than its current size the gap is filled with 0s.
* @param [in] size  New size of the file.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_sync()
* @brief This function is used in case we want to be sure that the physical file is synced with the OS buffers.
* @return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::secure_create(char *path, std::shared_ptr<SEFILE_HANDLE> hFile, int mode)
* @brief This function creates a new secure file managed with SEfile. If the file already exists, it is overwritten
*        with an empty one, all previous data are lost.
* @param [in] path Specify the absolute/relative path where to create the file. No encrypted directory are allowed inside the path.
* @param [out] hFile The pointer in which the file handle to the new opened file is placed after a success, NULL in case of failure.
* @param [in] mode The mode in which the file should be created. See \ref Mode_Defines.
* @return The function returns 0 in case of success. See \ref errorValues for error list.
* @details You do not need to call this function explicitly. Use secure_open() instead. */

/** \fn uint16_t SEfile::secure_key_check(uint16_t direction)
* @brief This function is used to check if the key setup in the SEfile object can be used to encrypt or decrypt data.
* @param [in] direction Check if the key can be used for encryption or decryption.
* @return The function returns 0 in case of success. See \ref errorValues for error list. This function works as SEfile::secure_getfilesize(). */

/** \fn uint16_t SEfile::get_filesize(uint32_t *length)
* @brief This function is used to compute the total logic size of a file that is already open within a SEfile object.
* @param [out] length Where the logic size of the file is stored.
* @return The function returns 0 in case of success. See \ref errorValues for error list. This function works as SEfile::secure_getfilesize(). */

/** \fn uint16_t SEfile::crypt_header(void *buff1, void *buff2, size_t datain_len, uint16_t direction)
* \brief This function encrypts a header buffer by exploiting the functions provided by \ref L1.h.
* \param [in] buff1 Pointer to the header we want to encrypt/decrypt.
* \param [out] buff2 Pointer to an allocated header where to store the result.
* \param [in] datain_len How big is the amount of data.
* \param [in] direction See \ref SE3_DIR.
* \return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::crypt_sectors(void *buff_decrypt, void *buff_crypt, size_t datain_len, size_t current_offset, uint8_t* nonce_ctr, uint8_t* nonce_pbkdf2)
*  \brief This function encrypts the buff_decrypt data by exploiting the functions provided by \ref L1.h.
*  \param [in] buff_decrypt The plaintext data to be encrypted
*  \param [out] buff_crypt  The preallocated buffer where to store the encrypted data.
*  \param [in] datain_len Specify how many data we want to encrypt.
*  \param [in] current_offset Current position inside the file expressed as number of cipher blocks
*  \param [in] nonce_ctr Initialization vector, see \ref SEFILE_HEADER
*  \param [in] nonce_pbkdf2 Initialization vector, see \ref SEFILE_HEADER
*  \return The function returns 0 in case of success. See \ref errorValues for error list. */

/** \fn uint16_t SEfile::decrypt_sectors(void *buff_crypt, void *buff_decrypt, size_t datain_len, size_t current_offset, uint8_t* nonce_ctr, uint8_t* nonce_pbkdf2)
*  \brief This function decrypts the buff_crypt data by exploiting the functions provided by \ref L1.h.
*  \param [in] buff_crypt The cipher text data to be decrypted
*  \param [out] buff_decrypt  The preallocated buffer where to store the decrypted data.
*  \param [in] datain_len Specify how many data we want to decrypt.
*  \param [in] current_offset Current position inside the file expressed as number of cipher blocks
*  \param [in] nonce_ctr Initialization vector, see \ref SEFILE_HEADER
*  \param [in] nonce_pbkdf2 Initialization vector, see \ref SEFILE_HEADER
*  \return The function returns 0 in case of success. See \ref errorValues for error list. */
